@@ -1,7 +1,7 @@
 #  File src/library/stats/R/models.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2021 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -195,7 +195,9 @@ reformulate <- function (termlabels, response=NULL, intercept = TRUE, env = pare
 	else
 	    call("~",
 		 ## response can be a symbol or call as  Surv(ftime, case)
-		 if(is.character(response))
+		 if(is.character(response)) {
+		     if(length(response) != 1)
+			 stop(gettextf("'%s' must be a character string", "response"), domain=NA)
                      tryCatch(str2lang(response),
                               error = function(e) {
                                   sc <- sys.calls()
@@ -210,6 +212,7 @@ reformulate <- function (termlabels, response=NULL, intercept = TRUE, env = pare
                                       call = reformCall)) # , domain=NA
                                   as.symbol(response)
                               })
+		 }
                  else response,
 		 terms)
     formula(fexpr, env)
@@ -575,9 +578,8 @@ model.frame.default <-
                              domain = NA)
 		    data[[nm]] <- factor(xi, levels=xl, exclude=NULL)
 		    if (!identical(attr(data[[nm]], "contrasts"), ctr))
-		    	warning(gettext(sprintf("contrasts dropped from factor %s",
-						nm), domain = NA),
-		    	        call. = FALSE)
+		    	warning(gettextf("contrasts dropped from factor %s", nm),
+		    	        call. = FALSE, domain = NA)
 		}
 	    }
     } else if(drop.unused.levels) {
@@ -588,9 +590,9 @@ model.frame.default <-
 	        ctr <- attr(x, "contrasts")
 		data[[nm]] <- x[, drop = TRUE]
 		if (!identical(attr(data[[nm]], "contrasts"), ctr))
-		    warning(gettext(sprintf(
+		    warning(gettextf(
 				"contrasts dropped from factor %s due to missing levels",
-					    nm), domain = NA), call. = FALSE)
+					    nm), domain = NA, call. = FALSE)
 	    }
 	}
     }
@@ -601,16 +603,16 @@ model.frame.default <-
 
 ## we don't assume weights are numeric or a vector, leaving this to the
 ## calling application
-model.weights <- function(x) x$"(weights)"
+model.weights <- function(x) .subset2(x, "(weights)")
 
 ## we do check that offsets are numeric.
 model.offset <- function(x) {
     offsets <- attr(attr(x, "terms"),"offset")
     if(length(offsets)) {
-	ans <- x$"(offset)" %||% 0
+	ans <- .subset2(x, "(offset)") %||% 0
 	for(i in offsets) ans <- ans+x[[i]]
     }
-    else ans <- x$"(offset)"
+    else ans <- .subset2(x, "(offset)")
     if(!is.null(ans) && !is.numeric(ans)) stop("'offset' must be numeric")
     ans
 }
@@ -660,7 +662,7 @@ model.matrix.default <- function(object, data = environment(object),
                 else {
                     ca <- contrasts.arg[[nn]]
                     if(is.matrix(ca)) contrasts(data[[ni]], ncol(ca)) <- ca
-                    else contrasts(data[[ni]]) <- contrasts.arg[[nn]]
+                    else contrasts(data[[ni]]) <- ca
                 }
             }
           }
