@@ -1,7 +1,7 @@
 #  File src/library/tools/R/sotools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2011-2020 The R Core Team
+#  Copyright (C) 2011-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ read_symbols_from_object_file <- function(f)
     if(!nzchar(nm)) {
         ## reasonable to assume nm is on the path
         nm <- Sys.which("nm")
-        if(!nzchar(nm)) nm <- shQuote(nm)
+        if(nzchar(nm)) nm <- shQuote(nm)
     }
     if(!nzchar(nm)) {
         warning("this requires 'nm' to be on the PATH")
@@ -105,13 +105,13 @@ system_ABI <- get_system_ABI()
 ## entry points for std::terminate are commented out as almost all
 ## come from system headers.
 so_symbol_names_table <-
-    ## 'linux' == glibc, principally
+    ## 'linux' == glibc, principally but checked with Alpine Linux's musl
     c("linux, C, gcc, abort, abort",
       ## https://refspecs.linuxbase.org/LSB_5.0.0/LSB-Core-generic/LSB-Core-generic/baselib---assert-fail-1.html
       "Linux, C, gcc, assert, __assert_fail",
       "linux, C, gcc, assert, __assert_fail_base",
       "linux, C, gcc, exit, exit",
-      "linux, C, gcc, _exit, _exit",
+      "linux, C, gcc, _exit, _exit", ## may not be seen
       "linux, C, gcc, _Exit, _Exit", ## _Exit is C99 and may not be a fn call
       "linux, C, gcc, printf, printf",
       "linux, C, gcc, printf, __printf_chk",
@@ -120,8 +120,13 @@ so_symbol_names_table <-
       "linux, C, gcc, putchar, putchar",
       "linux, C, gcc, stderr, stderr",
       "linux, C, gcc, stdout, stdout",
+      "linux, C, gcc, sprintf, sprintf",
+      "linux, C, gcc, sprintf, __sprintf_chk",
       "linux, C, gcc, vprintf, vprintf",
       "linux, C, gcc, vprintf, __vprintf_chk",
+      "linux, C, gcc, vsprintf, vsprintf",
+#      "linux, C, gcc, vprintf, vfprintf", # but also from REvprintf
+      "linux, C, gcc, vsprintf, __vsprintf_chk",
       "linux, C++, gxx, std::cout, _ZSt4cout",
       "linux, C++, gxx, std::cerr, _ZSt4cerr",
       #"linux, C++, gxx, std::terminate, _ZSt9terminatev",
@@ -133,7 +138,7 @@ so_symbol_names_table <-
       "linux, C, gcc, srandom_r, srandom_r",
       "linux, C, gcc, srand48, srand48",
       ## libc++ variants
-      "linux, C++, gxx, std::cout, _ZNSt3__14coutE",
+      "linux, C++, gxx, std::cout, _ZNSt3__14coutE", # std::__1::cout
       "linux, C++, gxx, std::cerr, _ZNSt3__14cerrE",
       "linux, Fortran, gfortran, open, _gfortran_st_open",
       "linux, Fortran, gfortran, close, _gfortran_st_close",
@@ -144,6 +149,18 @@ so_symbol_names_table <-
       "linux, Fortran, gfortran, stop, _gfortran_stop_numeric_f08",
       "linux, Fortran, gfortran, stop, _gfortran_stop_string",
       "linux, Fortran, gfortran, rand, _gfortran_rand",
+      "linux, Fortran, gfortran, random_init, _gfortran_random_init",
+      "linux, Fortran, gfortran, random_number, _gfortran_arandom_r4",
+      "linux, Fortran, gfortran, random_number, _gfortran_arandom_r8",
+      "linux, Fortran, gfortran, random_number, _gfortran_arandom_r16",
+      "linux, Fortran, gfortran, random_number, _gfortran_random_r4",
+      "linux, Fortran, gfortran, random_number, _gfortran_random_r8",
+      "linux, Fortran, gfortran, random_number, _gfortran_random_r16",
+      "linux, Fortran, gfortran, random_number, _gfortran_rand",
+      "linux, Fortran, gfortran, random_seed, _gfortran_random_seed_i4",
+      "linux, Fortran, gfortran, random_seed, _gfortran_random_seed_i8",
+
+      ## Classic flang from Dec 2017 (and untested since)
       "linux, Fortran, flang, open, f90io_open03",
       "linux, Fortran, flang, open, f90io_open2003",
       "linux, Fortran, flang, close, f90io_close",
@@ -156,41 +173,141 @@ so_symbol_names_table <-
       "linux, Fortran, flang, stop, f90_stop08",
       "linux, Fortran, flang, rand, rand",
 
-      ## clang identifies itself as gcc, so configure has used that
-      "macos, C, gcc, abort, _abort",
-      "macos, C, gcc, assert, ___assert_rtn",
+      ## and for for the 'flang' in the LLVM tree, currently
+      ## with executable 'flang-new'.
+      ## This currently has static libs for its runtimes.
+      "linux, Fortran, flang-new, stop, _FortranAStopStatement",
+      "linux, Fortran, flang-new, stop, _FortranAStopStatementText",
+      "linux, Fortran, flang-new, open, _FortranAioBeginOpenUnit",
+      "linux, Fortran, flang-new, close, _FortranAioBeginClose",
+      "linux, Fortran, flang-new, rewind, _FortranAioBeginRewind",
+      "linux, Fortran, flang-new, read, _FortranAioInputAscii",
+      "linux, Fortran, flang-new, read, _FortranAioInputCharacter",
+      "linux, Fortran, flang-new, read, _FortranAioInputComplex32",
+      "linux, Fortran, flang-new, read, _FortranAioInputComplex64",
+      "linux, Fortran, flang-new, read, _FortranAioOutputExternalListInput",
+      "linux, Fortran, flang-new, read, _FortranAioInputInteger",
+      "linux, Fortran, flang-new, read, _FortranAioInputLogical",
+      "linux, Fortran, flang-new, read, _FortranAioInputNamelist",
+      "linux, Fortran, flang-new, read, _FortranAioInputReal32",
+      "linux, Fortran, flang-new, read, _FortranAioInputReal64",
+      "linux, Fortran, flang-new, read, _FortranAioInputUnformattedBlock",
+      "linux, Fortran, flang-new, print, _FortranAioOutputAscii",
+      "linux, Fortran, flang-new, print, _FortranAioOutputCharacter",
+      "linux, Fortran, flang-new, print, _FortranAioOutputComplex32",
+      "linux, Fortran, flang-new, print, _FortranAioOutputComplex64",
+      "linux, Fortran, flang-new, print, _FortranAioOutputExternalListOutput",
+      "linux, Fortran, flang-new, print, _FortranAioOutputInteger128",
+      "linux, Fortran, flang-new, print, _FortranAioOutputInteger16",
+      "linux, Fortran, flang-new, print, _FortranAioOutputInteger32",
+      "linux, Fortran, flang-new, print, _FortranAioOutputInteger64",
+      "linux, Fortran, flang-new, print, _FortranAioOutputInteger8",
+      "linux, Fortran, flang-new, print, _FortranAioOutputLogical",
+      "linux, Fortran, flang-new, print, _FortranAioOutputNamelist",
+      "linux, Fortran, flang-new, print, _FortranAioOutputReal32",
+      "linux, Fortran, flang-new, print, _FortranAioOutputReal64",
+      "linux, Fortran, flang-new, write, _FortranAioOutputAscii",
+      "linux, Fortran, flang-new, write, _FortranAioOutputCharacter",
+      "linux, Fortran, flang-new, write, _FortranAioOutputComplex32",
+      "linux, Fortran, flang-new, write, _FortranAioOutputComplex64",
+      "linux, Fortran, flang-new, write, _FortranAioOutputExternalListOutput",
+      "linux, Fortran, flang-new, write, _FortranAioOutputInteger128",
+      "linux, Fortran, flang-new, write, _FortranAioOutputInteger16",
+      "linux, Fortran, flang-new, write, _FortranAioOutputInteger32",
+      "linux, Fortran, flang-new, write, _FortranAioOutputInteger64",
+      "linux, Fortran, flang-new, write, _FortranAioOutputInteger8",
+      "linux, Fortran, flang-new, write, _FortranAioOutputLogical",
+      "linux, Fortran, flang-new, write, _FortranAioOutputNamelist",
+      "linux, Fortran, flang-new, write, _FortranAioOutputReal32",
+      "linux, Fortran, flang-new, write, _FortranAioOutputReal64",
+      "linux, Fortran, flang-new, write, _FortranAioOutputUnformatedBlock",
+      ## does not support rand()
+      ## https://discourse.llvm.org/t/support-for-gnu-fortran-extensions/69630
+      "linux, Fortran, flang-new, random_init, _FortranARandomInit",
+      "linux, Fortran, flang-new, random_number, _FortranARandomNumber",
+      "linux, Fortran, flang-new, random_seed, _FortranARandomSeed",
+      "linux, Fortran, flang-new, random_seed, _FortranARandomSeedGet",
+      "linux, Fortran, flang-new, random_seed, _FortranARandomSeedSize",
+
+      ## Intel 'Clasic' and 202x
+      "linux, Fortran, intel, stop, for_stop",
+      "linux, Fortran, intel, stop, for_stop_core",
+      "linux, Fortran, intel, stop, for_stop_core8",
+      "linux, Fortran, intel, stop, for_stop_core_impl",
+      "linux, Fortran, intel, stop, for_stop_core_int",
+      "linux, Fortran, intel, stop, for_stop_core_int8",
+      "linux, Fortran, intel, stop, for_stop_core_quiet",
+      "linux, Fortran, intel, stop, for_stop_core_quiet_int8",
+      "linux, Fortran, intel, print, for_write_seq_lis",
+      "linux, Fortran, intel, open, for_open",
+      "linux, Fortran, intel, open, for_open_args",
+      "linux, Fortran, intel, open, for_open_default",
+      "linux, Fortran, intel, open, for_open_key",
+      "linux, Fortran, intel, close, for_close",
+      "linux, Fortran, intel, rewind, for_rewind",
+      "linux, Fortran, intel, read, for_read_seq_lis",
+      "linux, Fortran, intel, read, for_read_seq_fmt",
+      "linux, Fortran, intel, write, for_write_seq_lis",
+      "linux, Fortran, intel, write, for_write_seq_fmt",
+      "linux, Fortran, intel, write, for_write_seq_nml",
+      ## does not support rand() except in module ifport
+      "linux, Fortran, intel, rand, rand_",
+      "linux, Fortran, intel, random_number, for_random_number",
+      "linux, Fortran, intel, random_number, for_random_number_single",
+      "linux, Fortran, intel, random_seed, for_random_seed_bit_size",
+      "linux, Fortran, intel, random_seed, for_random_seed_get",
+
+      ## Apple clang identifies itself as gcc, so configure has used that
+      "macos, C, gcc, abort, _abort", # not currently seen
+      "macos, C, gcc, assert, ___assert_rtn", # not currently seen
       "macos, C, gcc, exit, _exit",
       "macos, C, gcc, _exit, __exit",
       "macos, C, gcc, _Exit, __Exit",
+      "macos, C, gcc, _Exit, __exit",
       "macos, C, gcc, printf, _printf",
       "macos, C, gcc, printf, _puts",
       "macos, C, gcc, puts, _puts",
       "macos, C, gcc, putchar, _putchar",
       "macos, C, gcc, stderr, ___stderrp",
       "macos, C, gcc, stdout, ___stdoutp",
+      "macos, C, gcc, sprintf, _sprintf", # old
+      "macos, C, gcc, sprintf, ___sprintf_chk",
       "macos, C, gcc, vprintf, _vprintf",
-      "macos, C++, gxx, std::cout, __ZSt4cout",
-      "macos, C++, gxx, std::cerr, __ZSt4cerr",
-      #"macos, C++, gxx, std::terminate, __ZSt9terminatev",
+      "macos, C, gcc, vsprintf, _vsprintf", # old
+      "macos, C, gcc, vsprintf, ___vsprintf_chk",
       "macos, C, gcc, rand, _rand",
       "macos, C, gcc, random, _random",
       "macos, C, gcc, rand_r, _rand_r",
       "macos, C, gcc, srand, _srand",
       "macos, C, gcc, srandom, _srandom",
       "macos, C, gcc, srand48, _srand48",
-      ## libc++ variants
-      "macos, C++, gxx, std::cout, __ZNSt3__14coutE",
+      #"macos, C++, gxx, std::cout, __ZSt4cout", # not with clang
+      #"macos, C++, gxx, std::cerr, __ZSt4cerr",
+      "macos, C++, gxx, std::cout, __ZNSt3__14coutE", # std::__1::cout
       "macos, C++, gxx, std::cerr, __ZNSt3__14cerrE",
+      #"macos, C++, gxx, std::terminate, __ZSt9terminatev",
       "macos, Fortran, gfortran, open, __gfortran_st_open",
       "macos, Fortran, gfortran, close, __gfortran_st_close",
-      "macos, Fortran, gfortran, rewind, _gfortran_st_rewind",
+      "macos, Fortran, gfortran, rewind, __gfortran_st_rewind",
       "macos, Fortran, gfortran, read, __gfortran_st_read",
       "macos, Fortran, gfortran, write, __gfortran_st_write",
       "macos, Fortran, gfortran, print, __gfortran_st_write",
       "macos, Fortran, gfortran, stop, __gfortran_stop_numeric",
       "macos, Fortran, gfortran, stop, __gfortran_stop_string",
       "macos, Fortran, gfortran, rand, __gfortran_rand",
+      "macos, Fortran, gfortran, random_init, __gfortran_random_init",
+      "macos, Fortran, gfortran, random_number, __gfortran_arandom_r4",
+      "macos, Fortran, gfortran, random_number, __gfortran_arandom_r8",
+      "macos, Fortran, gfortran, random_number, __gfortran_arandom_r16",
+      "macos, Fortran, gfortran, random_number, __gfortran_random_r4",
+      "macos, Fortran, gfortran, random_number, __gfortran_random_r8",
+      "macos, Fortran, gfortran, random_number, __gfortran_random_r16",
+      "macos, Fortran, gfortran, random_number, __gfortran_rand",
+      "macos, Fortran, gfortran, random_seed, __gfortran_random_seed_i4",
+      "macos, Fortran, gfortran, random_seed, __gfortran_random_seed_i8",
 
+      ## This is old: freebsd defaults to clang these days, and
+      ## gfortran and (classic) flang are available (and 'f18' will be)
       "freebsd, C, gcc, abort, abort",
       "freebsd, C, gcc, assert, __assert",
       "freebsd, C, gcc, exit, exit",
@@ -202,9 +319,14 @@ so_symbol_names_table <-
       "freebsd, C, gcc, putchar, putchar",
       "freebsd, C, gcc, stderr, __stderrp",
       "freebsd, C, gcc, stdout, __stdoutp",
+      "freebsd, C, gcc, sprintf, sprintf",
       "freebsd, C, gcc, vprintf, vprintf",
+      "freebsd, C, gcc, vsprintf, vsprintf",
       "freebsd, C++, gxx, std::cout, _ZSt4cout",
       "freebsd, C++, gxx, std::cerr, _ZSt4cerr",
+      ## libc++ variants
+      "freebsd, C++, gxx, std::cout, _ZNSt3__14coutE",
+      "freebsd, C++, gxx, std::cerr, _ZNSt3__14cerrE",
       "freebsd, C, gcc, rand, rand",
       "freebsd, C, gcc, random, random",
       "freebsd, C, gcc, srand, srand",
@@ -229,7 +351,9 @@ so_symbol_names_table <-
       "solaris, C, solcc, printf, printf",
       "solaris, C, solcc, putchar, putchar",
       "solaris, C, solcc, puts, puts",
+      "solaris, C, solcc, sprintf, sprintf",
       "solaris, C, solcc, vprintf, vprintf",
+      "solaris, C, solcc, vsprintf, vsprintf",
       "solaris, C++, solCC, std::cout, __1cDstdEcout_",
       "solaris, C++, solCC, std::cerr, __1cDstdEcerr_",
       #"solaris, C++, solCC, std::terminate, _ZSt9terminatev",
@@ -264,7 +388,9 @@ so_symbol_names_table <-
       "solaris, C, gcc, printf, puts",
       "solaris, C, gcc, puts, puts",
       "solaris, C, gcc, putchar, putchar",
+      "solaris, C, gcc, sprintf, sprintf",
       "solaris, C, gcc, vprintf, vprintf",
+      "solaris, C, gcc, vsprintf, vsprintf",
       "solaris, C, gcc, rand, rand",
       "solaris, C, gcc, random, random",
       "solaris, C, gcc, rand_r, rand_r",
@@ -307,7 +433,9 @@ so_symbol_names_table <-
       "windows, C, gcc, printf, puts",
       "windows, C, gcc, puts, puts",
       "windows, C, gcc, putchar, putchar",
+      "windows, C, gcc, sprintf, sprintf",
       "windows, C, gcc, vprintf, vprintf",
+      "windows, C, gcc, vsprintf, vsprintf",
       ## Windows does not have (s)random
       "windows, C, gcc, rand, rand",
       "windows, C, gcc, rand_r, rand_r",
@@ -419,13 +547,15 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
             "locale2charset", "match5", "matherr",
             "max_contour_segments", "mbcsToUcs2", "memtrace_report",
             "parseError", "pythag_", "rs_", "rwarnc_",
-            "signrank_free", "tql2_", "tqlrat_", "tred1_", "tred2_",
-            "utf8locale", "wilcox_free", "yylloc",
+            "tql2_", "tqlrat_", "tred1_", "tred2_", "utf8locale", "yylloc",
+            "R_opendir", "R_readdir", "R_closedir",
+            # "signrank_free", "wilcox_free" are API only from 4.2.0
 
 ## Rinterface.h, Rembedded.h, R_ext/{RStartup,eventloop}.h
             "AllDevicesKilled", "R_CStackLimit", "R_CStackStart",
             "R_ClearerrConsole", "R_CleanTempDir", "R_Consolefile",
-            "R_DefParams", "R_DirtyImage", "R_GUIType", "R_GlobalContext",
+            "R_DefCallbacks", "R_DefParams", "R_DefParamsEx",
+            "R_DirtyImage", "R_GUIType", "R_GlobalContext",
             "R_HistoryFile", "R_HistorySize", "R_Home", "R_HomeDir",
             "R_InputHandlers", "R_Interactive", "R_Outputfile",
             "R_PolledEvents", "R_ReplDLLdo1", "R_ReplDLLinit",
@@ -441,6 +571,7 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
             "Rf_KillAllDevices", "Rf_endEmbeddedR", "Rf_initEmbeddedR",
             "Rf_initialize_R", "Rf_jump_to_toplevel", "Rf_mainloop",
             "SaveAction", "addInputHandler", "editorcleanall", "fpu_setup",
+            "freeRUser", "free_R_HOME",
             "getDLLVersion", "getInputHandler", "getRUser", "get_R_HOME",
             "getSelectedHandler", "initStdinHandler",
             "process_site_Renviron", "process_system_Renviron",
@@ -454,6 +585,9 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
             "ptr_do_selectlist", "readconsolecfg", "removeInputHandler",
             "run_Rmainloop", "setup_Rmainloop",
 
+## non-API and long deprecated in R_ext/RS.h (and as call_S in S.h)
+            "call_R",
+
 ## in the non-API header R_ext/Connections.h
             "R_new_custom_connection", "R_ReadConnection",
             "R_WriteConnection", "R_GetConnection")
@@ -462,6 +596,7 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
 ## future <- c("dqrcf_", "dqrdc2_", "dqrls_", "dqrqty_", "dqrqy_", "optif9")
 ## d1mach_ and i1mach_ are mentioned (since R 2.15.3) in R-exts.
 
+
 ## grDevices uses R_Home R_InputHandlers R_TempDir R_Visible R_cairoCdynload R_fopen R_gzclose R_gzgets R_gzopen R_isForkedChild Rf_envlength Rf_strIsASCII Rf_utf8towcs Rg_set_col_ptrs Ri18n_wcwidth addInputHandler do_X11 do_contourLines do_getGraphicsEventEnv do_getSnapshot do_playSnapshot do_saveplot locale2charset mbcsToUcs2 ptr_R_ProcessEvents
 
 ## graphics uses OutDec R_print Rf_EncodeComplex Rf_EncodeInteger Rf_EncodeLogical Rf_EncodeReal Rf_GPretty Rf_PrintDefaults Rf_envlength Rf_formatComplex Rf_formatReal baseRegisterIndex known_to_be_latin1 max_contour_segments
@@ -474,7 +609,7 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
 
 ## tcltk uses R_Consolefile R_GUIType R_InputHandlers R_Outputfile R_PolledEvents R_checkActivity R_runHandlers R_timeout_handler R_timeout_val R_wait_usec ptr_R_ClearerrConsole ptr_R_FlushConsole ptr_R_ReadConsole ptr_R_ResetConsole ptr_R_WriteConsole
 
-## tools uses RC_fopen R_FileExists R_NewHashedEnv R_ParseContext R_ParseContextLast R_ParseContextLine R_ParseError R_ParseErrorMsg R_SrcfileSymbol R_SrcrefSymbol Rconn_fgetc Rf_begincontext Rf_endcontext Rf_envlength Rf_mbrtowc Rf_strchr extR_HTTPDCreate extR_HTTPDStop getConnection parseError
+## tools uses RC_fopen R_FileExists R_NewHashedEnv R_ParseContext R_ParseContextLast R_ParseContextLine R_ParseError R_ParseErrorMsg R_SrcfileSymbol R_SrcrefSymbol Rconn_fgetc Rf_begincontext Rf_endcontext Rf_envlength Rf_mbrtowc Rf_strchr extR_HTTPDCreate extR_HTTPDStop getConnection parseError R_opendir R_readdir R_closedir
 
 ## utils uses R_ClearerrConsole R_FreeStringBuffer R_GUIType R_moduleCdynload R_print R_strtod4 Rconn_fgetc Rconn_printf Rdownload Rf_EncodeElement Rf_PrintDefaults Rf_begincontext Rf_con_pushback Rf_endcontext Rf_envlength Rf_sortVector Rsockclose Rsockconnect Rsocklisten Rsockopen Rsockread Rsockwrite Runzip UNIMPLEMENTED_TYPE csduplicated do_Rprof do_Rprofmem do_edit getConnection known_to_be_latin1 ptr_R_addhistory ptr_R_loadhistory ptr_R_savehistory ptr_do_dataentry ptr_do_dataviewer ptr_do_selectlist
 
@@ -803,7 +938,10 @@ function(dir)
     ## A few packages such as CDM use base::.Call
     ff_call_names <- c(".C", ".Call", ".Fortran", ".External",
                        "base::.C", "base::.Call",
-                       "base::.Fortran", "base::.External")
+                       "base::.Fortran", "base::.External",
+                       ## internal ones
+                       ".Call.graphics", ".External.graphics",
+                       ".External2")
 
     predicate <- function(e) {
         (length(e) > 1L) &&
@@ -1003,27 +1141,27 @@ function(nrdb, align = TRUE, include_declarations = FALSE)
         "*/")
 
     if(include_declarations) {
+        prepare <- function(nargs)
+            if(nargs > 0) paste(rep.int("void *", nargs), collapse=", ")
+            else "void"
         decls <- c(
             "/* FIXME: ",
             "   Check these declarations against the C/Fortran source code.",
             "*/",
             if(NROW(y <- nrdb$.C)) {
-                 args <- sapply(y$n, function(n) if(n >= 0)
-                                paste(rep.int("void *", n), collapse=", ")
+                 args <- sapply(y$n, function(n) if(n >= 0) prepare(n)
                                 else "/* FIXME */")
                 c("", "/* .C calls */",
                   paste0("extern void ", y$s, "(", args, ");"))
            },
             if(NROW(y <- nrdb$.Call)) {
-                args <- sapply(y$n, function(n) if(n >= 0)
-                               paste(rep.int("SEXP", n), collapse=", ")
+                args <- sapply(y$n, function(n) if(n >= 0) prepare(n)
                                else "/* FIXME */")
                c("", "/* .Call calls */",
                   paste0("extern SEXP ", y$s, "(", args, ");"))
             },
             if(NROW(y <- nrdb$.Fortran)) {
-                 args <- sapply(y$n, function(n) if(n >= 0)
-                                paste(rep.int("void *", n), collapse=", ")
+                 args <- sapply(y$n, function(n) if(n >= 0) prepare(n)
                                 else "/* FIXME */")
                 c("", "/* .Fortran calls */",
                   paste0("extern void F77_NAME(", y$s, ")(", args, ");"))
