@@ -1,7 +1,7 @@
 #  File src/library/utils/R/objects.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2020 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ function()
     c(names(.knownS3Generics), tools:::.get_internal_S3_generics())
 
 .S3methods <-
-function(generic.function, class, envir=parent.frame())
+function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
 {
     rbindSome <- function(df, nms, msg) {
         ## rbind.data.frame() -- dropping rows with duplicated names
@@ -89,13 +89,9 @@ function(generic.function, class, envir=parent.frame())
     S3MethodsStopList <- tools::nonS3methods(NULL)
     knownGenerics <- getKnownS3generics()
     sp <- search()
-    if(nzchar(lookup <-
-                  Sys.getenv("_R_S3_METHOD_LOOKUP_BASEENV_AFTER_GLOBALENV_"))) {
-        lookup <- tools:::config_val_to_logical(lookup)
-        if(lookup) sp <- sp[c(1L, length(sp))]
-    }
+    if(dropPath) sp <- sp[c(1L, length(sp))]
     methods.called <- identical(sys.call(-1)[[1]], as.symbol("methods"))
-    an <- lapply(seq_along(sp), ls)
+    an <- lapply(sp, ls)
     lens <- lengths(an)
     an <- unlist(an, use.names=FALSE)
     names(an) <- rep.int(sp, lens)
@@ -108,7 +104,7 @@ function(generic.function, class, envir=parent.frame())
 	    generic.function <- deparse1(substitute(generic.function))
         ## else
         if(!exists(generic.function, mode = "function", envir = envir) &&
-           !any(generic.function == c("Math", "Ops", "Complex", "Summary")))
+           !any(generic.function == c("Math", "Ops", "Complex", "Summary", "matrixOps")))
             stop(gettextf("no function '%s' is visible", generic.function),
                  domain = NA)
         warn.not.generic <- FALSE
@@ -212,7 +208,7 @@ function(generic.function, class, envir=parent.frame())
 }
 
 methods <-
-function(generic.function, class)
+function(generic.function, class, dropPath = FALSE)
 {
     envir <- parent.frame()
     if(!missing(generic.function) && !is.character(generic.function)) {
@@ -231,7 +227,7 @@ function(generic.function, class)
     if (!missing(class) && !is.character(class))
         class <- deparse1(substitute(class))
 
-    s3 <- .S3methods(generic.function, class, envir)
+    s3 <- .S3methods(generic.function, class, envir, dropPath=dropPath)
     s4 <- if(.isMethodsDispatchOn()) methods::.S4methods(generic.function, class)
 
     .MethodsFunction(s3, s4, missing(generic.function))
@@ -272,6 +268,8 @@ print.MethodsFunction <- function(x, byclass = attr(x, "byclass"), ...)
 
 getS3method <- function(f, class, optional = FALSE, envir = parent.frame())
 {
+    stopifnot(is.character(f), length(f) == 1L)
+    stopifnot(is.character(class), length(class) == 1L)
     if(!any(f == getKnownS3generics())) {
         truegf <- findGeneric(f, envir)
         if(nzchar(truegf)) f <- truegf
@@ -313,7 +311,7 @@ isS3method <- function(method, f, class, envir = parent.frame())
     } else { # determine (f, class) from 'method'
 	f.c <- strsplit(method, ".", fixed=TRUE)[[1]]
 	nfc <- length(f.c)
-	if(nfc < 2 || !is.character(f.c))
+	if(nfc < 2 || !is.character(f.c) || f.c[[1L]] == "")
 	    return(FALSE) ## stop("Invalid 'method' specification; must be  \"<fun>.<class>\"")
 	if(nfc == 2) {
 	    f     <- f.c[[1L]]
@@ -610,14 +608,7 @@ function(x)
 function(genfun)
 {
     if (typeof(genfun) == "closure") {
-        lookup <- Sys.getenv("_R_S3_METHOD_LOOKUP_USE_TOPENV_AS_DEFENV_",
-                             "TRUE")
-        lookup <- tools:::config_val_to_logical(lookup)
-        if(lookup) {
-            topenv(environment(genfun))
-        } else {
-            environment(genfun)
-        }
+        topenv(environment(genfun))
     }
     else .BaseNamespaceEnv
 }
