@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997-2020  The R Core Team
- *  Copyright (C) 2003-2019  The R Foundation
+ *  Copyright (C) 1997-2023  The R Core Team
+ *  Copyright (C) 2003-2023  The R Foundation
  *  Copyright (C) 1995,1996  Robert Gentleman, Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -155,10 +155,9 @@ IntegerFromComplex(Rcomplex x, int *warn)
 int attribute_hidden
 IntegerFromString(SEXP x, int *warn)
 {
-    double xdouble;
-    char *endp;
     if (x != R_NaString && !isBlankString(CHAR(x))) { /* ASCII */
-	xdouble = R_strtod(CHAR(x), &endp); /* ASCII */
+	char *endp;
+	double xdouble = R_strtod(CHAR(x), &endp); /* ASCII */
 	if (isBlankString(endp)) {
 #ifdef _R_pre_Version_3_3_0
 	    if (xdouble > INT_MAX) {
@@ -194,10 +193,7 @@ RealFromLogical(int x, int *warn)
 double attribute_hidden
 RealFromInteger(int x, int *warn)
 {
-    if (x == NA_INTEGER)
-	return NA_REAL;
-    else
-	return x;
+    return (x == NA_INTEGER) ? NA_REAL : x;
 }
 
 double attribute_hidden
@@ -205,8 +201,6 @@ RealFromComplex(Rcomplex x, int *warn)
 {
     if (ISNAN(x.r) || ISNAN(x.i))
 	return NA_REAL;
-    if (ISNAN(x.r)) return x.r;
-    if (ISNAN(x.i)) return NA_REAL;
     if (x.i != 0)
 	*warn |= WARN_IMAG;
     return x.r;
@@ -304,7 +298,7 @@ ComplexFromString(SEXP x, int *warn)
     return z;
 }
 
-SEXP attribute_hidden StringFromLogical(int x, int *warn)
+attribute_hidden SEXP StringFromLogical(int x, int *warn)
 {
     int w;
     formatLogical(&x, 1, &w);
@@ -316,7 +310,7 @@ SEXP attribute_hidden StringFromLogical(int x, int *warn)
 #define SFI_CACHE_SIZE 512
 static SEXP sficache = NULL;
 
-SEXP attribute_hidden StringFromInteger(int x, int *warn)
+attribute_hidden SEXP StringFromInteger(int x, int *warn)
 {
     if (x == NA_INTEGER) return NA_STRING;
     else if (x >= 0 && x < SFI_CACHE_SIZE) {
@@ -325,7 +319,7 @@ SEXP attribute_hidden StringFromInteger(int x, int *warn)
 	    R_PreserveObject(sficache);
 	}
 	SEXP cval = STRING_ELT(sficache, x);
-	if (cval == R_BlankString) {
+	if (cval == R_BlankString) { // ""
 	    int w;
 	    formatInteger(&x, 1, &w);
 	    cval = mkChar(EncodeInteger(x, w));
@@ -342,7 +336,7 @@ SEXP attribute_hidden StringFromInteger(int x, int *warn)
 
 // dropTrailing0 and StringFromReal moved to printutils.c
 
-SEXP attribute_hidden StringFromComplex(Rcomplex x, int *warn)
+attribute_hidden SEXP StringFromComplex(Rcomplex x, int *warn)
 {
     int wr, dr, er, wi, di, ei;
     formatComplex(&x, 1, &wr, &dr, &er, &wi, &di, &ei, 0);
@@ -355,7 +349,7 @@ SEXP attribute_hidden StringFromComplex(Rcomplex x, int *warn)
 static SEXP StringFromRaw(Rbyte x, int *warn)
 {
     char buf[3];
-    sprintf(buf, "%02x", x);
+    snprintf(buf, 3, "%02x", x);
     return mkChar(buf);
 }
 
@@ -1179,8 +1173,18 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 
     SEXP ans = R_NilValue;	/* -Wall */
     if (ALTREP(v)) {
+	PROTECT(v); /* the methods should protect, but ... */
+	            /* also "v" is protected by caller */
 	ans = ALTREP_COERCE(v, type);
-	if (ans) return ans;
+	if (ans) {
+	    /* attribute copying could be handled by a Coerce_Ex
+	       method as for Duplicate; for now, do it here */
+	    PROTECT(ans);
+	    SHALLOW_DUPLICATE_ATTRIB(ans, v);
+	    UNPROTECT(2); /* v, ans */
+	    return ans;
+	}
+	UNPROTECT(1); /* v */
     }
 
     /* code to allow classes to extend ENVSXP, SYMSXP, etc */
@@ -1192,6 +1196,7 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	  return vv;
 	v = vv;
     }
+    PROTECT(v);
 
     switch (TYPEOF(v)) {
 #ifdef NOTYET
@@ -1204,7 +1209,10 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	break;
     case NILSXP:
     case LISTSXP:
-	if(type == LISTSXP) return v; // as coercePairList() is also used for LANGSXP
+	if(type == LISTSXP) {
+	    UNPROTECT(1); /* v */
+	    return v; // as coercePairList() is also used for LANGSXP
+	}
 	ans = coercePairList(v, type);
 	break;
     case LANGSXP: {
@@ -1220,7 +1228,7 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	PROTECT(ans = allocVector(type, n));
 	if (n == 0) {
 	    /* Can this actually happen? */
-	    UNPROTECT(1);
+	    UNPROTECT(1); /* ans */
 	    break;
 	}
 	int i = 0;
@@ -1244,7 +1252,7 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	    else
 		SET_STRING_ELT(ans, i, STRING_ELT(deparse1line(CAR(vp), 0), 0));
 	}
-	UNPROTECT(1);
+	UNPROTECT(1); /* ans */
 	break;
     }
     case VECSXP:
@@ -1284,7 +1292,9 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 		switch(TYPEOF(v)) {
 		case INTSXP:
 		case REALSXP:
-		    return R_deferred_coerceToString(v, NULL);
+		    ans = R_deferred_coerceToString(v, NULL);
+		    UNPROTECT(1); /* v */
+		    return ans;
 		}
 	    ans = coerceToString(v);	    break;
 	case EXPRSXP:
@@ -1300,6 +1310,7 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
     default:
 	COERCE_ERROR;
     }
+    UNPROTECT(1); /* v */
     return ans;
 }
 #undef COERCE_ERROR
@@ -1393,7 +1404,7 @@ static SEXP ascommon(SEXP call, SEXP u, SEXPTYPE type)
     return u;/* -Wall */
 }
 
-SEXP attribute_hidden do_asCharacterFactor(SEXP call, SEXP op, SEXP args,
+attribute_hidden SEXP do_asCharacterFactor(SEXP call, SEXP op, SEXP args,
                                            SEXP rho)
 {
     SEXP x;
@@ -1431,7 +1442,7 @@ SEXP asCharacterFactor(SEXP x)
 }
 
 
-SEXP attribute_hidden do_asatomic(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_asatomic(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x;
 
@@ -1478,13 +1489,24 @@ SEXP attribute_hidden do_asatomic(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
+/* _R_IS_AS_VECTOR_EXPERIMENTS_ : aiming for is.vector(as.vector(.))
+   consistency; specified at *start* of R session : */
+static int do_is_as_vector_experiments = -1;
+#define MAYBE_CACHE_DO_IS_AS_VECTORS_EXPERI				\
+do {									\
+  if(do_is_as_vector_experiments == -1) {				\
+    char *vector_experi = getenv("_R_IS_AS_VECTOR_EXPERIMENTS_");	\
+    do_is_as_vector_experiments =					\
+ 	((vector_experi != NULL) && StringTrue(vector_experi)) ? 1 : 0;	\
+  }									\
+} while(0)
+
+
 /* NB: as.vector is used for several other as.xxxx, including
    as.expression, as.list, as.pairlist, as.symbol, (as.single) */
-SEXP attribute_hidden do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP x, ans;
-    int type;
-
+    SEXP ans;
     /* DispatchOrEval internal generic: as.vector */
     if (DispatchOrEval(call, op, "as.vector", args, rho, &ans, 0, 1))
 	return(ans);
@@ -1493,14 +1515,14 @@ SEXP attribute_hidden do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* run the generic internal code */
 
     checkArity(op, args);
-    x = CAR(args);
-
     if (!isString(CADR(args)) || LENGTH(CADR(args)) != 1)
 	error_return(R_MSG_mode);
-    if (!strcmp("function", (CHAR(STRING_ELT(CADR(args), 0))))) /* ASCII */
-	type = CLOSXP;
-    else
-	type = str2type(CHAR(STRING_ELT(CADR(args), 0))); /* ASCII */
+
+    SEXP x = CAR(args);
+    int type =
+	(!strcmp("function", CHAR(STRING_ELT(CADR(args), 0))))  /* ASCII */
+	? CLOSXP
+	: str2type(CHAR(STRING_ELT(CADR(args), 0))); /* ASCII */
 
     /* "any" case added in 2.13.0 */
     if(type == ANYSXP || TYPEOF(x) == type) {
@@ -1516,8 +1538,21 @@ SEXP attribute_hidden do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    CLEAR_ATTRIB(ans);
 	    return ans;
 	case EXPRSXP:
-	case VECSXP:
-	    return x;
+	case VECSXP: {
+	    MAYBE_CACHE_DO_IS_AS_VECTORS_EXPERI;
+	    if(do_is_as_vector_experiments) {
+		if(ATTRIB(x) == R_NilValue) return x;
+		if(OBJECT(x)) return x; // protect e.g.  setClass(., contains="list")
+		SEXP nms = getAttrib(x, R_NamesSymbol);
+		if(nms != R_NilValue && CDR(ATTRIB(x)) == R_NilValue) return x;
+		ans = MAYBE_REFERENCED(x) ? duplicate(x) : x;
+		CLEAR_ATTRIB(ans);
+		if (nms != R_NilValue)
+		    setAttrib(ans, R_NamesSymbol, nms);
+		return ans;
+	    } else
+		return x;
+	}
 	default:
 	    ;
 	}
@@ -1563,7 +1598,7 @@ SEXP attribute_hidden do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
-SEXP attribute_hidden do_asfunction(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_asfunction(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP arglist, envir, names, pargs, body;
     int i, n;
@@ -1629,7 +1664,7 @@ static void parse_cleanup(void *data)
 /* primitive,
  * op = 0 : str2lang(s)
  * op = 1 : str2expression(text) */
-SEXP attribute_hidden do_str2lang(SEXP call, SEXP op, SEXP args, SEXP rho) {
+attribute_hidden SEXP do_str2lang(SEXP call, SEXP op, SEXP args, SEXP rho) {
     checkArity(op, args);
     // check1arg(args, call, "s");
     args = CAR(args);
@@ -1693,7 +1728,7 @@ SEXP attribute_hidden do_str2lang(SEXP call, SEXP op, SEXP args, SEXP rho) {
 }
 
 /* primitive */
-SEXP attribute_hidden do_ascall(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_ascall(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -1740,27 +1775,17 @@ SEXP attribute_hidden do_ascall(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
-/* call and rho are only needed for _R_CHECK_LENGTH_1_LOGIC2_ checking
-       and diagnostics; to be removed if length>1 value is turned to error */
 /* return int, not Rboolean, for NA_LOGICAL : */
-int asLogical2(SEXP x, int checking, SEXP call, SEXP rho)
+int asLogical2(SEXP x, int checking, SEXP call)
 {
     int warn = 0;
 
     if (isVectorAtomic(x)) {
 	if (XLENGTH(x) < 1)
 	    return NA_LOGICAL;
-	if (checking && XLENGTH(x) > 1) {
-	    char msg[128];
-	    snprintf(msg, 128, _("'length(x) = %lld > 1' in coercion to '%s'"),
-		    (long long) XLENGTH(x), "logical(1)");
-	    R_BadValueInRCode(x, call, rho,
-		"length > 1 in coercion to logical",
-		msg,
-		msg,
-		"_R_CHECK_LENGTH_1_LOGIC2_",
-		FALSE /* by default do nothing */);
-	}
+	if (checking && XLENGTH(x) > 1)
+	    errorcall(call, _("'length = %lld' in coercion to '%s'"),
+		      (long long) XLENGTH(x), "logical(1)");
 	switch (TYPEOF(x)) {
 	case LGLSXP:
 	    return LOGICAL_ELT(x, 0);
@@ -1785,7 +1810,7 @@ int asLogical2(SEXP x, int checking, SEXP call, SEXP rho)
 
 int asLogical(SEXP x)
 {
-    return asLogical2(x, /* checking = */ 0, R_NilValue, R_NilValue);
+    return asLogical2(x, /* checking = */ 0, R_NilValue);
 }
 
 
@@ -1930,7 +1955,7 @@ Rcomplex asComplex(SEXP x)
 
 
 /* return the type (= "detailed mode") of the SEXP */
-SEXP attribute_hidden do_typeof(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_typeof(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     return type2rstr(TYPEOF(CAR(args)));
@@ -1939,7 +1964,7 @@ SEXP attribute_hidden do_typeof(SEXP call, SEXP op, SEXP args, SEXP rho)
 /* Define many of the <primitive> "is.xxx" functions :
    Note that  isNull, isNumeric, etc are defined in util.c or ../include/Rinlinedfuns.h
 */
-SEXP attribute_hidden do_is(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_is(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     checkArity(op, args);
@@ -2110,24 +2135,22 @@ SEXP attribute_hidden do_is(SEXP call, SEXP op, SEXP args, SEXP rho)
  */
 
 // is.vector(x, mode) :
-SEXP attribute_hidden do_isvector(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_isvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP ans, a, x;
-    const char *stype;
-
     checkArity(op, args);
-    x = CAR(args);
+    SEXP x = CAR(args);
     if (!isString(CADR(args)) || LENGTH(CADR(args)) != 1)
 	error_return(R_MSG_mode);
 
-    stype = CHAR(STRING_ELT(CADR(args), 0)); /* ASCII */
+    const char *stype = CHAR(STRING_ELT(CADR(args), 0)); // 'mode' in R ; ASCII
 
     /* "name" and "symbol" are synonymous */
     if (streql(stype, "name"))
       stype = "symbol";
 
-    PROTECT(ans = allocVector(LGLSXP, 1));
-    if (streql(stype, "any")) {
+    SEXP ans = PROTECT(allocVector(LGLSXP, 1));
+    Rboolean any = streql(stype, "any");
+    if (any) {
 	/* isVector is inlined, means atomic or VECSXP or EXPRSXP */
 	LOGICAL0(ans)[0] = isVector(x);
     }
@@ -2142,9 +2165,25 @@ SEXP attribute_hidden do_isvector(SEXP call, SEXP op, SEXP args, SEXP rho)
     else
 	LOGICAL0(ans)[0] = 0;
 
-    /* We allow a "names" attribute on any vector. */
-    if (LOGICAL0(ans)[0] && ATTRIB(CAR(args)) != R_NilValue) {
-	a = ATTRIB(CAR(args));
+    if (LOGICAL0(ans)[0]) {
+      Rboolean IS_vector = FALSE;
+      MAYBE_CACHE_DO_IS_AS_VECTORS_EXPERI;
+      if(do_is_as_vector_experiments) {
+	if((IS_vector = any && isVectorList(x) && OBJECT(x))) {
+	    // use of is.vector(.) to check for non-matrix/array => try dim(.) :
+	  static SEXP op_dim = NULL;
+	  if (op_dim == NULL)
+	      op_dim = R_Primitive("dim");
+	  SEXP args = PROTECT(list1(x));
+	  IS_vector = isNull(do_dim(call, op_dim, args, rho));
+	  UNPROTECT(1);
+	}
+      }
+      if(IS_vector) {
+	  // list or expression w/ is.object() and no dim(): stay TRUE
+      } else if (ATTRIB(x) != R_NilValue) {
+	/* We allow a "names" attribute on any vector. */
+	SEXP a = ATTRIB(x);
 	while(a != R_NilValue) {
 	    if (TAG(a) != R_NamesSymbol) {
 		LOGICAL0(ans)[0] = 0;
@@ -2152,6 +2191,7 @@ SEXP attribute_hidden do_isvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    }
 	    a = CDR(a);
 	}
+      }
     }
     UNPROTECT(1);
     return (ans);
@@ -2181,7 +2221,7 @@ static R_INLINE void copyDimAndNames(SEXP x, SEXP ans)
     }
 }
 
-SEXP attribute_hidden do_isna(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_isna(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x;
     R_xlen_t i, n;
@@ -2396,7 +2436,7 @@ static Rboolean anyNA(SEXP call, SEXP op, SEXP args, SEXP env)
     return FALSE;
 } // anyNA()
 
-SEXP attribute_hidden do_anyNA(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_anyNA(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     static SEXP do_anyNA_formals = NULL;
@@ -2427,7 +2467,7 @@ SEXP attribute_hidden do_anyNA(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
-SEXP attribute_hidden do_isnan(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_isnan(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x;
     R_xlen_t i, n;
@@ -2476,10 +2516,11 @@ SEXP attribute_hidden do_isnan(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-SEXP attribute_hidden do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, names, dims;
     R_xlen_t i, n;
+    int nprotect = 0;
 
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -2494,6 +2535,7 @@ SEXP attribute_hidden do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
     x = CAR(args);
     n = xlength(x);
     PROTECT(ans = allocVector(LGLSXP, n));
+    nprotect++;
     int *pa = LOGICAL(ans);
     if (isVector(x)) {
 	dims = getAttrib(x, R_DimSymbol);
@@ -2501,6 +2543,7 @@ SEXP attribute_hidden do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    PROTECT(names = getAttrib(x, R_DimNamesSymbol));
 	else
 	    PROTECT(names = getAttrib(x, R_NamesSymbol));
+	nprotect++;
     }
     else dims = names = R_NilValue;
     switch (TYPEOF(x)) {
@@ -2537,17 +2580,16 @@ SEXP attribute_hidden do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 	else
 	    setAttrib(ans, R_NamesSymbol, names);
     }
-    if (isVector(x))
-	UNPROTECT(1); /* names */
-    UNPROTECT(1); /* ans */
+    UNPROTECT(nprotect);
     return ans;
 }
 
-SEXP attribute_hidden do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, names, dims;
     double xr, xi;
     R_xlen_t i, n;
+    int nprotect = 0;
 
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -2562,6 +2604,7 @@ SEXP attribute_hidden do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
     x = CAR(args);
     n = xlength(x);
     PROTECT(ans = allocVector(LGLSXP, n));
+    nprotect++;
     int *pa = LOGICAL(ans);
     if (isVector(x)) {
 	dims = getAttrib(x, R_DimSymbol);
@@ -2569,6 +2612,7 @@ SEXP attribute_hidden do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    PROTECT(names = getAttrib(x, R_DimNamesSymbol));
 	else
 	    PROTECT(names = getAttrib(x, R_NamesSymbol));
+	nprotect++;
     }
     else	dims = names = R_NilValue;
     switch (TYPEOF(x)) {
@@ -2612,14 +2656,12 @@ SEXP attribute_hidden do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 	else
 	    setAttrib(ans, R_NamesSymbol, names);
     }
-    if (isVector(x))
-	UNPROTECT(1); /* names */
-    UNPROTECT(1); /* ans */
+    UNPROTECT(nprotect);
     return ans;
 }
 
 /* This is a primitive SPECIALSXP */
-SEXP attribute_hidden do_call(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_call(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP rest, evargs, rfun, tmp;
 
@@ -2645,7 +2687,7 @@ SEXP attribute_hidden do_call(SEXP call, SEXP op, SEXP args, SEXP rho)
     return (rfun);
 }
 
-SEXP attribute_hidden do_docall(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_docall(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP c, fun, names, envir;
     int i, n;
@@ -2657,7 +2699,7 @@ SEXP attribute_hidden do_docall(SEXP call, SEXP op, SEXP args, SEXP rho)
     envir = CADDR(args);
     args = CADR(args);
 
-    /* must be a string or a function:
+    /* must be a string or a function
        zero-length string check used to be here but install gives
        better error message.
      */
@@ -2756,7 +2798,7 @@ SEXP substitute(SEXP lang, SEXP rho)
 /* Work through a list doing substitute on the
    elements taking particular care to handle '...' */
 
-SEXP attribute_hidden substituteList(SEXP el, SEXP rho)
+attribute_hidden SEXP substituteList(SEXP el, SEXP rho)
 {
     SEXP h, p = R_NilValue, res = R_NilValue;
 
@@ -2809,7 +2851,7 @@ SEXP attribute_hidden substituteList(SEXP el, SEXP rho)
 
 
 /* This is a primitive SPECIALSXP */
-SEXP attribute_hidden do_substitute(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_substitute(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP argList, env, s, t;
     static SEXP do_substitute_formals = NULL;
@@ -2843,7 +2885,7 @@ SEXP attribute_hidden do_substitute(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 /* This is a primitive SPECIALSXP */
-SEXP attribute_hidden do_quote(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_quote(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     check1arg(args, call, "expr");
@@ -2898,6 +2940,9 @@ static int class2type(const char *s)
 
 static SEXP do_unsetS4(SEXP obj, SEXP newClass)
 {
+  // act now, as warning()s may be treated as errors:
+  UNSET_S4_OBJECT(obj);
+
   if(isNull(newClass))  { /* NULL class is only valid for S3 objects */
     warning(_("Setting class(x) to NULL;   result will no longer be an S4 object"));
   }
@@ -2908,7 +2953,6 @@ static SEXP do_unsetS4(SEXP obj, SEXP newClass)
   else
     warning(_("Setting class(x) to \"%s\" sets attribute to NULL; result will no longer be an S4 object"),
 	    CHAR(asChar(newClass)));
-  UNSET_S4_OBJECT(obj);
   return obj;
 }
 
@@ -2931,17 +2975,30 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	PROTECT(dup = duplicate(value));
 	PROTECT(value = coerceVector(dup, STRSXP));
 	nProtect += 2;
+	if(length(value) == 0)
+	    error(_("invalid replacement object to be a class string"));
     }
-    if(length(value) > 1) {
-	setAttrib(obj, R_ClassSymbol, value);
+    // length(value) >= 1
+    const char *valueString = CHAR(asChar(value)); /* ASCII ; the *first* in case of multiple strings */
+    if(!strcmp("matrix", valueString)) { // value : just "matrix" or  c("matrix", "<some>" [, ...])
+	if(length(getAttrib(obj, R_DimSymbol)) != 2)
+	    error(_("invalid to set the class to matrix unless the dimension attribute is of length 2 (was %d)"),
+		  length(getAttrib(obj, R_DimSymbol)));
+	if(length(value) == 1 ||
+	   (length(value) == 2 && !strcmp("array", CHAR(STRING_ELT(value, 1)))))
+	{ // "matrix"  or  c("matrix", "array") :
+	    setAttrib(obj, R_ClassSymbol, R_NilValue); // NULL;  and that's not for S4:
+	    if(IS_S4_OBJECT(obj))
+		do_unsetS4(obj, value);
+	    goto done_set_class;
+	}
+    }
+
+    if(length(value) > 1) { // multiple strings ==> S3
+        setAttrib(obj, R_ClassSymbol, value);
 	if(IS_S4_OBJECT(obj)) /*  multiple strings only valid for S3 objects */
-	  do_unsetS4(obj, value);
-    }
-    else if(length(value) == 0) {
-	error(_("invalid replacement object to be a class string"));
-    }
-    else {
-	const char *valueString = CHAR(asChar(value)); /* ASCII */
+	    do_unsetS4(obj, value);
+    } else { // length(value) == 1 -- use valueString
 	int whichType = class2type(valueString);
 	SEXPTYPE valueType = (whichType == -1) ? (SEXPTYPE) -1
 	    : classTable[whichType].sexp;
@@ -2972,14 +3029,6 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	}
 	/* the next 2 special cases mirror the special code in
 	 * R_data_class */
-	else if(!strcmp("matrix", valueString)) {
-	    if(length(getAttrib(obj, R_DimSymbol)) != 2)
-		error(_("invalid to set the class to matrix unless the dimension attribute is of length 2 (was %d)"),
-		 length(getAttrib(obj, R_DimSymbol)));
-	    setAttrib(obj, R_ClassSymbol, R_NilValue);
-	    if(IS_S4_OBJECT(obj))
-	      do_unsetS4(obj, value);
-	}
 	else if(!strcmp("array", valueString)) {
 	    if(length(getAttrib(obj, R_DimSymbol)) <= 0)
 		error(_("cannot set class to \"array\" unless the dimension attribute has length > 0"));
@@ -2992,40 +3041,38 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	    setAttrib(obj, R_ClassSymbol, value);
 	}
     }
+
+done_set_class:
     UNPROTECT(nProtect);
     return obj;
 }
 
-SEXP attribute_hidden R_do_set_class(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP R_do_set_class(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans;
     checkArity(op, args);
     check1arg(args, call, "x");
 
-    if (MAYBE_SHARED(CAR(args))) SETCAR(args, shallow_duplicate(CAR(args)));
-    ans = R_set_class(CAR(args), CADR(args), call);
+    if (MAYBE_SHARED(CAR(args)) ||
+	((! IS_ASSIGNMENT_CALL(call)) && MAYBE_REFERENCED(CAR(args))))
+	SETCAR(args, shallow_duplicate(CAR(args)));
+    SEXP ans = R_set_class(CAR(args), CADR(args), call);
     SETTER_CLEAR_NAMED(CAR(args));
     return ans;
 }
 
-/* primitive */
-SEXP attribute_hidden do_storage_mode(SEXP call, SEXP op, SEXP args, SEXP env)
+/* primitive:  storage.mode(obj) <- value  */
+attribute_hidden SEXP do_storage_mode(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-/* storage.mode(obj) <- value */
-    SEXP obj, value, ans;
-    SEXPTYPE type;
-
     checkArity(op, args);
     check1arg(args, call, "x");
 
-    obj = CAR(args);
-
-    value = CADR(args);
+    SEXP
+      obj = CAR(args),
+      value = CADR(args);
     if (!isValidString(value) || STRING_ELT(value, 0) == NA_STRING)
 	error(_("'value' must be non-null character string"));
-    type = str2type(CHAR(STRING_ELT(value, 0)));
+    SEXPTYPE type = str2type(CHAR(STRING_ELT(value, 0)));
     if(type == (SEXPTYPE) -1) {
-	/* For backwards compatibility we allow "real" and "single" */
 	if(streql(CHAR(STRING_ELT(value, 0)), "real")) {
 	    error("use of 'real' is defunct: use 'double' instead");
 	} else if(streql(CHAR(STRING_ELT(value, 0)), "single")) {
@@ -3036,8 +3083,8 @@ SEXP attribute_hidden do_storage_mode(SEXP call, SEXP op, SEXP args, SEXP env)
     if(TYPEOF(obj) == type) return obj;
     if(isFactor(obj))
 	error(_("invalid to change the storage mode of a factor"));
-    PROTECT(ans = coerceVector(obj, type));
-    SHALLOW_DUPLICATE_ATTRIB(ans, obj);
+    SEXP ans = PROTECT(coerceVector(obj, type));
+    SHALLOW_DUPLICATE_ATTRIB(ans, obj); // keeping attributes plus OBJECT & S4 bits
     UNPROTECT(1);
     return ans;
 }
