@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2019  The R Core Team
+ *  Copyright (C) 2000-2022  The R Core Team
  *  Copyright (C) 2005       The R Foundation
  *  Copyright (C) 1995-1997  Robert Gentleman and Ross Ihaka
  *
@@ -72,7 +72,7 @@
 #define NINTERRUPT 10000000
 
 
-SEXP attribute_hidden complex_unary(ARITHOP_TYPE code, SEXP s1, SEXP call)
+attribute_hidden SEXP complex_unary(ARITHOP_TYPE code, SEXP s1, SEXP call)
 {
     R_xlen_t i, n;
     SEXP ans;
@@ -169,7 +169,7 @@ static double complex mycpow (double complex X, double complex Y)
 
 
 
-SEXP attribute_hidden complex_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
+attribute_hidden SEXP complex_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 {
     R_xlen_t i, i1, i2, n, n1, n2;
     SEXP ans;
@@ -242,7 +242,7 @@ SEXP attribute_hidden complex_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
     return ans;
 }
 
-SEXP attribute_hidden do_cmathfuns(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_cmathfuns(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, y = R_NilValue;	/* -Wall*/
     R_xlen_t i, n;
@@ -357,7 +357,7 @@ SEXP attribute_hidden do_cmathfuns(SEXP call, SEXP op, SEXP args, SEXP env)
 
 /* Implementing  signif(<complex>)  *and* used in format.c and printutils.c */
 #define MAX_DIGITS 22
-void attribute_hidden z_prec_r(Rcomplex *r, const Rcomplex *x, double digits)
+attribute_hidden void z_prec_r(Rcomplex *r, const Rcomplex *x, double digits)
 {
     // Implement    r <- signif(x, digits)
 
@@ -602,7 +602,7 @@ static Rboolean cmath1(double complex (*f)(double complex),
     return naflag;
 }
 
-SEXP attribute_hidden complex_math1(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP complex_math1(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, y;
     R_xlen_t n;
@@ -690,7 +690,7 @@ static void z_atan2(Rcomplex *r, Rcomplex *csn, Rcomplex *ccs)
 	/* Complex Functions of Two Arguments */
 
 typedef void (*cm2_fun)(Rcomplex *, Rcomplex *, Rcomplex *);
-SEXP attribute_hidden complex_math2(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP complex_math2(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     R_xlen_t i, n, na, nb, ia, ib;
     Rcomplex ai, bi, *y;
@@ -748,7 +748,7 @@ SEXP attribute_hidden complex_math2(SEXP call, SEXP op, SEXP args, SEXP env)
     return sy;
 }
 
-SEXP attribute_hidden do_complex(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_complex(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     /* complex(length, real, imaginary) */
     SEXP ans, re, im;
@@ -789,7 +789,7 @@ SEXP attribute_hidden do_complex(SEXP call, SEXP op, SEXP args, SEXP rho)
 static void R_cpolyroot(double *opr, double *opi, int *degree,
 			double *zeror, double *zeroi, Rboolean *fail);
 
-SEXP attribute_hidden do_polyroot(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_polyroot(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP z, zr, zi, r, rr, ri;
     Rboolean fail;
@@ -1123,7 +1123,7 @@ static Rboolean fxshft(int l2, double *zr, double *zi)
  * Uses global (sr,si), nn, pr[], pi[], .. (all args of polyev() !)
 */
 
-    Rboolean pasd, bool, test;
+    Rboolean pasd, h_s_0, test;
     static double svsi, svsr;
     static int i, j, n;
     static double oti, otr;
@@ -1139,7 +1139,7 @@ static Rboolean fxshft(int l2, double *zr, double *zi)
 
     /* calculate first t = -p(s)/h(s). */
 
-    calct(&bool);
+    calct(&h_s_0);
 
     /* main loop for one second stage step. */
 
@@ -1150,15 +1150,15 @@ static Rboolean fxshft(int l2, double *zr, double *zi)
 
 	/* compute next h polynomial and new t. */
 
-	nexth(bool);
-	calct(&bool);
+	nexth(h_s_0);
+	calct(&h_s_0);
 	*zr = sr + tr;
 	*zi = si + ti;
 
 	/* test for convergence unless stage 3 has */
 	/* failed once or this is the last h polynomial. */
 
-	if (!bool && test && j != l2) {
+	if (!h_s_0 && test && j != l2) {
 	    if (hypot(tr - otr, ti - oti) >= hypot(*zr, *zi) * 0.5) {
 		pasd = FALSE;
 	    }
@@ -1194,7 +1194,7 @@ static Rboolean fxshft(int l2, double *zr, double *zi)
 		sr = svsr;
 		si = svsi;
 		polyev(nn, sr, si, pr, pi, qpr, qpi, &pvr, &pvi);
-		calct(&bool);
+		calct(&h_s_0);
 	    }
 	}
     }
@@ -1218,7 +1218,7 @@ static Rboolean vrshft(int l3, double *zr, double *zi)
  *
  * Assign and uses  GLOBAL sr, si
 */
-    Rboolean bool, b;
+    Rboolean h_s_0, b;
     static int i, j;
     static double r1, r2, mp, ms, tp, relstp;
     static double omp;
@@ -1263,8 +1263,8 @@ static Rboolean vrshft(int l3, double *zr, double *zi)
 		sr = r2;
 		polyev(nn, sr, si, pr, pi, qpr, qpi, &pvr, &pvi);
 		for (j = 1; j <= 5; ++j) {
-		    calct(&bool);
-		    nexth(bool);
+		    calct(&h_s_0);
+		    nexth(h_s_0);
 		}
 		omp = infin;
 		goto L10;
@@ -1283,10 +1283,10 @@ static Rboolean vrshft(int l3, double *zr, double *zi)
 	/* calculate next iterate. */
 
     L10:
-	calct(&bool);
-	nexth(bool);
-	calct(&bool);
-	if (!bool) {
+	calct(&h_s_0);
+	nexth(h_s_0);
+	calct(&h_s_0);
+	if (!h_s_0) {
 	    relstp = hypot(tr, ti) / hypot(sr, si);
 	    sr += tr;
 	    si += ti;
@@ -1300,10 +1300,10 @@ L_conv:
     return TRUE;
 }
 
-static void calct(Rboolean *bool)
+static void calct(Rboolean *h_s_0)
 {
     /* computes	 t = -p(s)/h(s).
-     * bool   - logical, set true if h(s) is essentially zero.	*/
+     * h_s_0   - logical, set true if h(s) is essentially zero.	*/
 
     int n = nn - 1;
     double hvi, hvr;
@@ -1312,8 +1312,8 @@ static void calct(Rboolean *bool)
     polyev(n, sr, si, hr, hi,
 	   qhr, qhi, &hvr, &hvi);
 
-    *bool = hypot(hvr, hvi) <= are * 10. * hypot(hr[n-1], hi[n-1]);
-    if (!*bool) {
+    *h_s_0 = hypot(hvr, hvi) <= are * 10. * hypot(hr[n-1], hi[n-1]);
+    if (!*h_s_0) {
 	cdivid(-pvr, -pvi, hvr, hvi, &tr, &ti);
     }
     else {
@@ -1322,15 +1322,15 @@ static void calct(Rboolean *bool)
     }
 }
 
-static void nexth(Rboolean bool)
+static void nexth(Rboolean h_s_0)
 {
     /* calculates the next shifted h polynomial.
-     * bool :	if TRUE  h(s) is essentially zero
+     * h_s_0 :	if TRUE  h(s) is essentially zero
      */
     int j, n = nn - 1;
     double t1, t2;
 
-    if (!bool) {
+    if (!h_s_0) {
 	for (j=1; j < n; j++) {
 	    t1 = qhr[j - 1];
 	    t2 = qhi[j - 1];
