@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2018  The R Core Team.
+ *  Copyright (C) 1998-2023  The R Core Team.
  *  Copyright (C) 1995-1998  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -138,31 +138,27 @@ static SEXP seq_colon(double n1, double n2, SEXP call)
     return ans;
 }
 
-SEXP attribute_hidden do_colon(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_colon(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP s1, s2;
-    double n1, n2;
-
     checkArity(op, args);
-    if (inherits(CAR(args), "factor") && inherits(CADR(args), "factor"))
-	return(cross_colon(call, CAR(args), CADR(args)));
+    SEXP s1 = CAR(args),
+	s2 = CADR(args);
+    if (inherits(s1, "factor") && inherits(s2, "factor"))
+	return(cross_colon(call, s1, s2));
+    double
+	n1 = length(s1),
+	n2 = length(s2);
+    if (n1 != 1 || n2 != 1) {
+	if (n1 == 0 || n2 == 0)
+	    errorcall(call, _("argument of length 0"));
+	char *check = getenv("_R_CHECK_LENGTH_COLON_");
+	if (check ? StringTrue(check) : FALSE) // warn by default
+	    errorcall(call, _("numerical expression has length > 1"));
+	else
+	    warningcall(call, _("numerical expression has %d elements: only the first used"),
+			(n1 > 1) ? (int) n1 : (int) n2);
+    }
 
-    s1 = CAR(args);
-    s2 = CADR(args);
-    n1 = length(s1);
-    n2 = length(s2);
-    if (n1 == 0 || n2 == 0)
-	errorcall(call, _("argument of length 0"));
-    if (n1 > 1)
-	warningcall(call,
-		    ngettext("numerical expression has %d element: only the first used",
-			     "numerical expression has %d elements: only the first used",
-			     (int) n1), (int) n1);
-    if (n2 > 1)
-	warningcall(call,
-		    ngettext("numerical expression has %d element: only the first used",
-			     "numerical expression has %d elements: only the first used",
-			     (int) n2), (int) n2);
     n1 = asReal(s1);
     n2 = asReal(s2);
     if (ISNAN(n1) || ISNAN(n2))
@@ -339,16 +335,20 @@ static SEXP rep3(SEXP s, R_xlen_t ns, R_xlen_t na)
 }
 
 // .Internal(rep.int(x, times))
-SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP s = CAR(args), ncopy = CADR(args);
     R_xlen_t nc;
     SEXP a;
 
+    /* DispatchOrEval internal generic: rep.int */
     if (DispatchOrEval(call, op, "rep.int", args, rho, &a, 0, 0))
       return(a);
 
+    if (DispatchOrEval(call, op, "rep", args, rho, &a, 0, 0))
+      return(a);
+
     if (!isVector(ncopy))
 	error(_("invalid type (%s) for '%s' (must be a vector)"),
 	      type2char(TYPEOF(ncopy)), "times");
@@ -399,18 +399,32 @@ SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
     return a;
 }
 
-SEXP attribute_hidden do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     R_xlen_t ns, na;
     SEXP a, s, len;
 
     checkArity(op, args);
 
+    /* DispatchOrEval internal generic: rep_len */
     if (DispatchOrEval(call, op, "rep_len", args, rho, &a, 0, 0))
       return(a);
 
     s = CAR(args);
 
+    if (isObject(s)) {
+	SEXP rep_call;
+	PROTECT(rep_call = shallow_duplicate(call));
+	SETCAR(rep_call, install("rep"));
+	SET_TAG(CDDR(rep_call), install("length.out"));
+	SET_TAG(CDR(args), install("length.out"));
+	if (DispatchOrEval(rep_call, op, "rep", args, rho, &a, 0, 0)) {
+	    UNPROTECT(1);
+	    return(a);
+	}
+	UNPROTECT(1);
+    }
+
     if (!isVector(s) && s != R_NilValue)
 	error(_("attempt to replicate non-vector"));
 
@@ -615,13 +629,14 @@ static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, R_xlen_t each, R_xlen_t nt)
    rep(1:3,,8) matches length.out */
 
 /* This is a primitive SPECIALSXP with internal argument matching */
-SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, times = R_NilValue;
     R_xlen_t i, lx, len = NA_INTEGER, each = 1, nt;
     static SEXP do_rep_formals = NULL;
 
     /* includes factors, POSIX[cl]t, Date */
+    /* DispatchOrEval internal generic: rep */
     if (DispatchOrEval(call, op, "rep", args, rho, &ans, 0, 0))
 	return(ans);
 
@@ -636,7 +651,7 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 	do_rep_formals = allocFormalsList5(install("x"), install("times"),
 					   install("length.out"),
 					   install("each"), R_DotsSymbol);
-    PROTECT(args = matchArgs(do_rep_formals, args, call));
+    PROTECT(args = matchArgs_NR(do_rep_formals, args, call));
 
     x = CAR(args);
     /* supported in R 2.15.x */
@@ -779,14 +794,13 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
  */
 #define FEPS 1e-10
 /* to match seq.default */
-SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans = R_NilValue /* -Wall */, from, to, by, len, along;
-    int nargs = length(args), lf;
-    Rboolean One = (nargs == 1);
     R_xlen_t i, lout = NA_INTEGER;
-    static SEXP do_seq_formals = NULL;
+    Rboolean One = (length(args) == 1); // *before* messing with args ..
 
+    /* DispatchOrEval internal generic: seq.int */
     if (DispatchOrEval(call, op, "seq", args, rho, &ans, 0, 1))
 	return(ans);
 
@@ -794,11 +808,12 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
        We pretend this is
        seq(from, to, by, length.out, along.with, ...)
     */
+    static SEXP do_seq_formals = NULL;
     if (do_seq_formals == NULL)
 	do_seq_formals = allocFormalsList6(install("from"), install("to"),
 					   install("by"), install("length.out"),
 					   install("along.with"), R_DotsSymbol);
-    PROTECT(args = matchArgs(do_seq_formals, args, call));
+    PROTECT(args = matchArgs_NR(do_seq_formals, args, call));
 
     from = CAR(args); args = CDR(args);
     to   = CAR(args); args = CDR(args);
@@ -808,9 +823,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
     Rboolean
 	miss_from = (from == R_MissingArg),
 	miss_to   = (to   == R_MissingArg);
-
     if(One && !miss_from) {
-	lf = length(from);
+	int lf = length(from);
 	if(lf == 1 && (TYPEOF(from) == INTSXP || TYPEOF(from) == REALSXP)) {
 	    double rfrom = asReal(from);
 	    if (!R_FINITE(rfrom))
@@ -840,7 +854,7 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
 
     if(lout == NA_INTEGER) {
-	double rfrom, rto, rby = asReal(by);
+	double rfrom, rto;
 	if(miss_from) rfrom = 1.0;
 	else {
 	    if(length(from) != 1) errorcall(call, _("'%s' must be of length 1"), "from");
@@ -857,15 +871,20 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	if(by == R_MissingArg)
 	    ans = seq_colon(rfrom, rto, call);
-	else {
+	else { // 'by' specified
 	    if(length(by) != 1) errorcall(call, _("'%s' must be of length 1"), "by");
 	    double del = rto - rfrom;
 	    if(del == 0.0 && rto == 0.0) {
 		ans = to; // is *not* missing in this case
 		goto done;
 	    }
-	    /* printf("from = %f, to = %f, by = %f\n", rfrom, rto, rby); */
-	    double n = del/rby;
+	    double n, rby = asReal(by);
+	    Rboolean finite_del = R_FINITE(del);
+	    if(finite_del) {
+		n = del/rby;
+	    } else { // overflow in  (to - from)  when both are finite
+		n = rto/rby - rfrom/rby;
+	    }
 	    if(!R_FINITE(n)) {
 		if(del == 0.0 && rby == 0.0) {
 		    ans = miss_from ? ScalarReal(rfrom) : from;
@@ -873,8 +892,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		} else
 		    errorcall(call, _("invalid '(to - from)/by'"));
 	    }
-	    double dd = fabs(del)/fmax2(fabs(rto), fabs(rfrom));
-	    if(dd < 100 * DBL_EPSILON) {
+	    // inherited from seq.default() but "fudgy"
+	    if(finite_del && fabs(del)/fmax2(fabs(rto), fabs(rfrom)) < 100 * DBL_EPSILON) {
 		ans = miss_from ? ScalarReal(rfrom) : from;
 		goto done;
 	    }
@@ -889,8 +908,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 	    R_xlen_t nn;
 	    if((miss_from || TYPEOF(from) == INTSXP) &&
-	       (miss_to   || TYPEOF(to)   == INTSXP) &&
-	       TYPEOF(by) == INTSXP) {
+	       (miss_to   || TYPEOF(to)   == INTSXP) && TYPEOF(by) == INTSXP)
+	    {
 		int *ia, ifrom = miss_from ? (int)rfrom : asInteger(from),
 		    iby = asInteger(by);
 		/* With the current limits on integers and FEPS
@@ -908,8 +927,15 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		nn = (int)(n + FEPS);
 		ans = allocVector(REALSXP, nn+1);
 		double *ra = REAL(ans);
-		for(i = 0; i <= nn; i++)
-		    ra[i] = rfrom + (double)i * rby;
+		if(finite_del)
+		    for(i = 0; i <= nn; i++)
+			ra[i] = rfrom + (double)i * rby;
+		else { // |from - to| is infinite, but n = (from-to)/by is not
+		    rfrom /= 4.;
+		    rby   /= 4.;
+		    for(i = 0; i <= nn; i++) // ldexp(Y, 2) := Y * 2^2 = 4 Y
+			ra[i] = ldexp(rfrom + (double)i * rby, 2);
+		}
 		/* Added in 2.9.0 */
 		if (nn > 0)
 		    if((rby > 0 && ra[nn] > rto) || (rby < 0 && ra[nn] < rto))
@@ -920,20 +946,27 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	ans = allocVector(INTSXP, 0);
     } else if (One) {
 	ans = seq_colon(1.0, (double)lout, call);
-    } else if (by == R_MissingArg) { // and  len := length.out  specified
+    } else if (by == R_MissingArg) { // and  len := length.out  specified, >= 1
 	double rfrom = asReal(from), rto = asReal(to), rby = 0; // -Wall
 	if(miss_to)   rto   = rfrom + (double)lout - 1;
 	if(miss_from) rfrom = rto   - (double)lout + 1;
 	if(!R_FINITE(rfrom)) errorcall(call, _("'%s' must be a finite number"), "from");
 	if(!R_FINITE(rto))   errorcall(call, _("'%s' must be a finite number"), "to");
-	if(lout > 2) rby = (rto - rfrom)/(double)(lout - 1);
+	Rboolean finite_del = 0;
+	if(lout > 2) { // only then, use 'by'
+	    double nint = (double)(lout - 1);
+	    if((finite_del = R_FINITE(rby = (rto - rfrom))))
+		rby /= nint;
+	    else // overflow in (to - from), nint >= 2  => finite 'by'
+		rby = (rto/nint - rfrom/nint);
+	}
 	if(rfrom <= INT_MAX && rfrom >= INT_MIN &&
 	   rto   <= INT_MAX && rto   >= INT_MIN &&
 	   rfrom == (int)rfrom &&
 	   (lout <= 1 || rto == (int)rto) &&
 	   (lout <= 2 || rby == (int)rby)) {
 	    ans = allocVector(INTSXP, lout);
-	    if(lout > 0) INTEGER(ans)[0] = (int)rfrom;
+	    INTEGER(ans)[0] = (int)rfrom;
 	    if(lout > 1) INTEGER(ans)[lout - 1] = (int)rto;
 	    if(lout > 2)
 		for(i = 1; i < lout-1; i++) {
@@ -941,26 +974,35 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    INTEGER(ans)[i] = (int)(rfrom + (double)i*rby);
 		}
 	} else {
-	ans = allocVector(REALSXP, lout);
-	if(lout > 0) REAL(ans)[0] = rfrom;
-	if(lout > 1) REAL(ans)[lout - 1] = rto;
-	if(lout > 2) {
-	    rby = (rto - rfrom)/(double)(lout - 1);
-	    for(i = 1; i < lout-1; i++) {
-//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-		REAL(ans)[i] = rfrom + (double)i*rby;
+	    ans = allocVector(REALSXP, lout);
+	    REAL(ans)[0] = rfrom;
+	    if(lout > 1) REAL(ans)[lout - 1] = rto;
+	    if(lout > 2) {
+		if(finite_del)
+		    for(i = 1; i < lout-1; i++) {
+// 		        if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+			REAL(ans)[i] = rfrom + (double)i*rby;
+		    }
+		else {  // del:=(from - to) is infinite, but n = del/by is not
+		    rfrom /= 4.; // or ldexp(*, -2) for speed
+		    rby   /= 4.;
+		    for(i = 1; i < lout-1; i++) {
+// 		        if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+			REAL(ans)[i] = ldexp(rfrom + (double)i*rby, 2); // ldexp(y,2) = y * 4
+		    }
+		}
 	    }
 	}
-	}
     } else if (miss_to) {
 	double rfrom = asReal(from), rby = asReal(by), rto;
 	if(miss_from) rfrom = 1.0;
 	if(!R_FINITE(rfrom)) errorcall(call, _("'%s' must be a finite number"), "from");
 	if(!R_FINITE(rby))   errorcall(call, _("'%s' must be a finite number"), "by");
 	rto = rfrom + (double)(lout-1)*rby;
-	if(rby == (int)rby && rfrom == (int)rfrom
-	   && rfrom <= INT_MAX && rfrom >= INT_MIN
-	   &&  rto  <= INT_MAX &&  rto  >= INT_MIN) {
+	// avoid undefined behaviour by testing range before converting.
+	if(rfrom <= INT_MAX && rfrom >= INT_MIN
+	   &&  rto  <= INT_MAX &&  rto  >= INT_MIN
+	   && rby == (int)rby && rfrom == (int)rfrom) {
 	    ans = allocVector(INTSXP, lout);
 	    for(i = 0; i < lout; i++) {
 //		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
@@ -1001,7 +1043,7 @@ done:
     return ans;
 }
 
-SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     R_xlen_t len;
@@ -1037,7 +1079,7 @@ SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
 	return R_compact_intrange(1, len);
 }
 
-SEXP attribute_hidden do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     R_xlen_t len;
 
@@ -1065,3 +1107,66 @@ SEXP attribute_hidden do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
     else
 	return R_compact_intrange(1, len);
 }
+
+attribute_hidden SEXP do_sequence(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    R_xlen_t lengths_len, from_len, by_len, ans_len, i, i2, i3;
+    int from_elt, by_elt, length, j, k, *ans_elt;
+    const int *lengths_elt;
+    SEXP ans, lengths, from, by;
+
+    checkArity(op, args);
+
+    lengths = CAR(args);
+    if (!isInteger(lengths))
+	error(_("'lengths' is not of mode integer"));
+    from = CADR(args);
+    if (!isInteger(from))
+	error(_("'from' is not of mode integer"));
+    by = CADDR(args);
+    if (!isInteger(by))
+	error(_("'by' is not of mode integer"));
+
+    lengths_len = length(lengths);
+    from_len = length(from);
+    by_len = length(by);
+    if (lengths_len != 0) {
+	if (from_len == 0)
+	    error(_("'from' has length 0, but not 'lengths'"));
+	if (by_len == 0)
+	    error(_("'by' has length 0, but not 'lengths'"));
+    }
+    ans_len = 0;
+    lengths_elt = INTEGER(lengths);
+    for (i = 0; i < lengths_len; i++, lengths_elt++) {
+	length = *lengths_elt;
+	if (length == NA_INTEGER || length < 0)
+	    error(_("'lengths' must be a vector of non-negative integers"));
+	ans_len += length;
+    }
+    PROTECT(ans = allocVector(INTSXP, ans_len));
+    ans_elt = INTEGER(ans);
+    lengths_elt = INTEGER(lengths);
+    for (i = i2 = i3 = 0; i < lengths_len; i++, i2++, i3++, lengths_elt++) {
+	if (i2 >= from_len)
+	    i2 = 0; /* recycle */
+	if (i3 >= by_len)
+	    i3 = 0; /* recycle */
+	length = *lengths_elt;
+	from_elt = INTEGER(from)[i2];
+	if (length != 0 && from_elt == NA_INTEGER) {
+	    UNPROTECT(1);
+	    error(_("'from' contains NAs"));
+	}
+	by_elt = INTEGER(by)[i3];
+	if (length >= 2 && by_elt == NA_INTEGER) {
+	    UNPROTECT(1);
+	    error(_("'by' contains NAs"));
+	}
+	// int to = from_elt + (length - 1) * by_elt;
+	for (k = 0, j = from_elt; k < length; j += by_elt, k++)
+	    *(ans_elt++) = j;
+    }
+    UNPROTECT(1);
+    return ans;
+}
