@@ -1,5 +1,5 @@
 /*
- *  R : A Computer Langage for Statistical Data Analysis
+ *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *  Copyright (C) 2006-2016 The R Core Team
  *
@@ -22,11 +22,11 @@
 #include <config.h>
 #endif
 
-#include "Defn.h"
+#include <Defn.h>
 #include <Internal.h>
 #include <R_ext/Itermacros.h>
 
-SEXP attribute_hidden do_split(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_split(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, f, counts, vec, nm, nmj;
     Rboolean have_names;
