@@ -1,18 +1,16 @@
-#ifndef MATRIX_UNPAMATRIX_H
-#define MATRIX_UNPAMATRIX_H
+#ifndef MATRIX_UNPACKEDMATRIX_H
+#define MATRIX_UNPACKEDMATRIX_H
 
-#include "Lapack-etc.h"
 #include "Mutils.h"
 
-SEXP unpackedMatrix_pack(SEXP from, SEXP strict, SEXP tr_if_ge, SEXP up_if_ge);
 SEXP unpackedMatrix_force_symmetric(SEXP from, SEXP uplo_to);
 
-SEXP unpackedMatrix_is_symmetric(SEXP obj, SEXP checkDN);
-SEXP matrix_is_symmetric(SEXP obj, SEXP checkDN);
-
 SEXP unpackedMatrix_is_triangular(SEXP obj, SEXP upper);
 SEXP matrix_is_triangular(SEXP obj, SEXP upper);
 
+SEXP unpackedMatrix_is_symmetric(SEXP obj, SEXP checkDN);
+SEXP matrix_is_symmetric(SEXP obj, SEXP checkDN);
+
 SEXP unpackedMatrix_is_diagonal(SEXP obj);
 SEXP matrix_is_diagonal(SEXP obj);
 
@@ -26,4 +24,4 @@ SEXP matrix_symmpart(SEXP from);
 SEXP unpackedMatrix_skewpart(SEXP from);
 SEXP matrix_skewpart(SEXP from);
 
-#endif
+#endif /* MATRIX_UNPACKEDMATRIX_H */
